@@ -16,21 +16,16 @@
    Boston, MA 02110-1301, USA.
 */
 #include "Generator.h"
+#include <klocale.h>
 
-#include "kdeversion.h"
-#include <KLocale>
 #include <QFile>
 #include <QApplication>
-#include <QEventLoop>
-//Added by qt3to4:
-#include <Q3ValueList>
+#include <QList>
 #include "ImageManager/Manager.h"
 #include <KFileDialog>
-#include <KStandardDirs>
 #include <KRun>
 #include <KMessageBox>
 #include "Utilities/Util.h"
-#include "Utilities/UniqFilenameMapper.h"
 #include <KDebug>
 #include <QDir>
 #include "ImportExport/Export.h"
@@ -78,7 +73,7 @@ void HTMLGenerator::Generator::generate()
     _filenameMapper.reset();
 
     // Itertate over each of the image sizes needed.
-    for( Q3ValueList<ImageSizeCheckBox*>::ConstIterator sizeIt = _setup.activeResolutions().begin();
+     for( QList<ImageSizeCheckBox*>::ConstIterator sizeIt = _setup.activeResolutions().begin();
          sizeIt != _setup.activeResolutions().end(); ++sizeIt ) {
 
         bool ok = generateIndexPage( (*sizeIt)->width(), (*sizeIt)->height() );
@@ -166,6 +161,14 @@ bool HTMLGenerator::Generator::generateIndexPage( int width, int height )
 
     content.replace( QString::fromLatin1( "**DESCRIPTION**" ), _setup.description() );
     content.replace( QString::fromLatin1( "**TITLE**" ), _setup.title() );
+
+    QString copyright;
+    if (!_setup.copyright().isEmpty())
+        copyright = QString::fromLatin1( "&#169; %1" ).arg( _setup.copyright() );
+    else
+        copyright = QString::fromLatin1( "&nbsp;" );
+    content.replace( QString::fromLatin1( "**COPYRIGHT**" ), copyright );
+
     QString kimLink = QString::fromLatin1( "Share and Enjoy <a href=\"%1\">KPhotoAlbum export file</a>" ).arg( kimFileName( true ) );
     if ( _setup.generateKimFile() )
         content.replace( QString::fromLatin1( "**KIMFILE**" ), kimLink );
@@ -182,6 +185,15 @@ bool HTMLGenerator::Generator::generateIndexPage( int width, int height )
     // so we keep it using QDom.
     int count = 0;
     int cols = _setup.numOfCols();
+    int minWidth = 0;
+    int minHeight = 0;
+    QString first, last, images;
+
+    images += QString::fromLatin1( "var gallery=new Array()\nvar width=%1\nvar height=%2\nvar tsize=%3\n" ).arg( width ).arg( height ).arg( _setup.thumbSize() );
+
+    minImageSize(minWidth, minHeight);
+    images += QString::fromLatin1( "var minPage=\"index-%1x%2.html \"\n" ).arg( minWidth ).arg( minHeight );
+
     QDomElement row;
     Q_FOREACH(const DB::ImageInfoPtr info, _setup.imageList().fetchInfos()) {
         if ( wasCanceled() )
@@ -199,6 +211,30 @@ bool HTMLGenerator::Generator::generateIndexPage( int width, int height )
         row.appendChild( col );
 
         const QString fileName = info->fileName(DB::AbsolutePath);
+
+        if (first.isEmpty())
+            first = namePage( width, height, fileName);
+        else
+            last = namePage( width, height, fileName);
+
+	if (!Utilities::isVideo(fileName))
+            images += QString::fromLatin1( "gallery.push([\"%1\", \"%2\", \"%3\", \"" )
+                  .arg( nameImage( fileName, width ) ).arg( nameImage( fileName, _setup.thumbSize() ) ).arg( nameImage( fileName, maxImageSize() ) );
+	else
+            images += QString::fromLatin1( "gallery.push([\"%1\", \"%2\", \"%3\", \"" )
+                  .arg( nameImage( fileName, _setup.thumbSize() ) ).arg( nameImage( fileName, _setup.thumbSize() ) ).arg( QFileInfo(fileName).fileName() );
+
+        // -------------------------------------------------- Description
+        QString description = populateDescription(DB::ImageDB::instance()->categoryCollection()->categories(), info);
+
+        if ( !description.isEmpty() )
+            description = QString::fromLatin1 ( "<ul>%1</ul>" ).arg ( description );
+        else
+            description = QString::fromLatin1 ( "" );
+
+        images += description;
+        images += QString::fromLatin1( "\"]);\n" );
+
         QDomElement href = doc.createElement( QString::fromLatin1( "a" ) );
         href.setAttribute( QString::fromLatin1( "href" ),
                            namePage( width, height, fileName));
@@ -215,6 +251,12 @@ bool HTMLGenerator::Generator::generateIndexPage( int width, int height )
 
     content.replace( QString::fromLatin1( "**THUMBNAIL-TABLE**" ), doc.toString() );
 
+    content.replace( QString::fromLatin1( "**JSIMAGES**" ), images );
+    if (!first.isEmpty())
+	content.replace( QString::fromLatin1( "**FIRST**" ), first );
+    if (!last.isEmpty())
+	content.replace( QString::fromLatin1( "**LAST**" ), last );
+
     // -------------------------------------------------- Resolutions
     QString resolutions;
     QList<ImageSizeCheckBox*> actRes = _setup.activeResolutions();
@@ -304,11 +346,22 @@ bool HTMLGenerator::Generator::generateContentPage( int width, int height,
         link = i18n( "prev" );
     content.replace( QString::fromLatin1( "**PREV**" ), link );
 
+    // PENDING(blackie) These next 5 line also exists exactly like that in HTMLGenerator::Generator::generateIndexPage. Please refactor.
+    // prevfile
+    if ( !prev.isNull() )
+        link = namePage( width, height, prev.fetchInfo()->fileName(DB::AbsolutePath));
+    else
+        link = i18n( "prev" );
+    content.replace( QString::fromLatin1( "**PREVFILE**" ), link );
 
     // index link
     link = i18n( "<a href=\"index-%1.html\">index</a>", ImageSizeCheckBox::text(width,height,true));
     content.replace( QString::fromLatin1( "**INDEX**" ), link );
 
+    // indexfile
+    link = QString::fromLatin1( "index-%1.html").arg(ImageSizeCheckBox::text(width,height,true));
+    content.replace( QString::fromLatin1( "**INDEXFILE**" ), link );
+
     // Next Link
     if ( !next.isNull() )
         link = i18n( "<a href=\"%1\">next</a>", namePage( width, height, next.fetchInfo()->fileName(DB::AbsolutePath)));
@@ -316,6 +369,13 @@ bool HTMLGenerator::Generator::generateContentPage( int width, int height,
         link = i18n( "next" );
     content.replace( QString::fromLatin1( "**NEXT**" ), link );
 
+    // Nextfile
+    if ( !next.isNull() )
+        link = namePage( width, height, next.fetchInfo()->fileName(DB::AbsolutePath));
+    else
+        link = i18n( "next" );
+    content.replace( QString::fromLatin1( "**NEXTFILE**" ), link );
+
     if ( !next.isNull() )
         link = namePage( width, height, next.fetchInfo()->fileName(DB::AbsolutePath) );
     else
@@ -326,9 +386,9 @@ bool HTMLGenerator::Generator::generateContentPage( int width, int height,
 
     // -------------------------------------------------- Resolutions
     QString resolutions;
-    const Q3ValueList<ImageSizeCheckBox*>& actRes = _setup.activeResolutions();
+     const QList<ImageSizeCheckBox*>& actRes = _setup.activeResolutions();
     if ( actRes.count() > 1 ) {
-        for( Q3ValueList<ImageSizeCheckBox*>::ConstIterator sizeIt = actRes.begin();
+         for( QList<ImageSizeCheckBox*>::ConstIterator sizeIt = actRes.begin();
              sizeIt != actRes.end(); ++sizeIt ) {
             int w = (*sizeIt)->width();
             int h = (*sizeIt)->height();
@@ -344,24 +404,18 @@ bool HTMLGenerator::Generator::generateContentPage( int width, int height,
     }
     content.replace( QString::fromLatin1( "**RESOLUTIONS**" ), resolutions );
 
+    // -------------------------------------------------- Copyright
+    QString copyright;
+
+    if ( !_setup.copyright().isEmpty() )
+        copyright = QString::fromLatin1( "&#169; %1" ).arg( _setup.copyright() );
+    else
+        copyright = QString::fromLatin1( "&nbsp;" );
+    content.replace( QString::fromLatin1( "**COPYRIGHT**" ), QString::fromLatin1( "%1" ).arg( copyright ) );
+
+
     // -------------------------------------------------- Description
-    QString description;
-
-    Q3ValueList<DB::CategoryPtr> categories = DB::ImageDB::instance()->categoryCollection()->categories();
-    for( Q3ValueList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it ) {
-        if ( (*it)->isSpecialCategory() )
-            continue;
-
-        QString name = (*it)->name();
-        if ( !info->itemsOfCategory( name ).empty() && _setup.includeCategory(name) ) {
-            QString val = QStringList(info->itemsOfCategory( name ).toList()).join( QString::fromLatin1(", ") );
-            description += QString::fromLatin1("  <li> <b>%1:</b> %2\n").arg( name ).arg( val );
-        }
-    }
-
-    if ( !info->description().isEmpty() && _setup.includeCategory( QString::fromLatin1( "**DESCRIPTION**" )) ) {
-        description += QString::fromLatin1( "  <li> <b>Description:</b> %1\n" ).arg( info->description() );
-    }
+    QString description = populateDescription(DB::ImageDB::instance()->categoryCollection()->categories(), info);
 
     if ( !description.isEmpty() )
         content.replace( QString::fromLatin1( "**DESCRIPTION**" ), QString::fromLatin1( "<ul>\n%1\n</ul>" ).arg( description ) );
@@ -546,13 +600,28 @@ void HTMLGenerator::Generator::getThemeInfo( QString* baseDir, QString* name, QS
 int HTMLGenerator::Generator::maxImageSize()
 {
     int res = 0;
-    for( Q3ValueList<ImageSizeCheckBox*>::ConstIterator sizeIt = _setup.activeResolutions().begin();
+     for( QList<ImageSizeCheckBox*>::ConstIterator sizeIt = _setup.activeResolutions().begin();
          sizeIt != _setup.activeResolutions().end(); ++sizeIt ) {
         res = qMax( res, (*sizeIt)->width() );
     }
     return res;
 }
 
+void HTMLGenerator::Generator::minImageSize(int& width, int& height)
+{
+    width = height = 0;
+    for( QList<ImageSizeCheckBox*>::ConstIterator sizeIt = _setup.activeResolutions().begin();
+         sizeIt != _setup.activeResolutions().end(); ++sizeIt ) {
+	if ((width == 0) && ((*sizeIt)->width() > 0)) {
+	    width = (*sizeIt)->width();
+	    height = (*sizeIt)->height();
+	} else if ((*sizeIt)->width() > 0) {
+            width = qMin( width, (*sizeIt)->width() );
+	    height = qMin( height, (*sizeIt)->height());
+	}
+    }
+}
+
 void HTMLGenerator::Generator::showBrowser()
 {
     if ( _setup.generateKimFile() )
@@ -565,5 +634,25 @@ void HTMLGenerator::Generator::showBrowser()
     _eventLoop.exit();
 }
 
+QString HTMLGenerator::Generator::populateDescription( QList<DB::CategoryPtr> categories, const DB::ImageInfoPtr info )
+{
+     QString description;
 
+     for( QList<DB::CategoryPtr>::Iterator it = categories.begin(); it != categories.end(); ++it ) {
+        if ( (*it)->isSpecialCategory() )
+            continue;
+
+        QString name = (*it)->name();
+        if ( !info->itemsOfCategory( name ).empty() && _setup.includeCategory(name) ) {
+            QString val = QStringList(info->itemsOfCategory( name ).toList()).join( QString::fromLatin1(", ") );
+            description += QString::fromLatin1("  <li> <b>%1:</b> %2</li>").arg( name ).arg( val );
+        }
+    }
+
+    if ( !info->description().isEmpty() && _setup.includeCategory( QString::fromLatin1( "**DESCRIPTION**" )) ) {
+        description += QString::fromLatin1( "  <li> <b>Description:</b> %1</li>" ).arg( info->description() );
+    }
+
+    return description;
+}
 #include "Generator.moc"
